package in.ocsf.frei.geld.core.model;/* kpmy 24.11.2017 */

import in.ocsf.frei.geld.core.model.aspects.DwarfAspect;
import in.ocsf.frei.geld.core.repo.WorldDwarfObjectRepository;
import in.ocsf.frei.geld.core.repo.WorldObjectRepository;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.context.ApplicationContext;
import org.springframework.dao.OptimisticLockingFailureException;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

import static in.ocsf.frei.geld.core.model.QWorld_Something.something;

public class DwarfContext {

    private ApplicationContext context;

    private List<Pair<DwarfAspect, ContextEvent>> events = new ArrayList<>();

    private Long hour;

    public DwarfContext(ApplicationContext context) {
        this.context = context;
    }

    public void addEvent(DwarfAspect aspect, ContextEvent event) {
        events.add(Pair.of(aspect, event));
    }

    public boolean hasSimpleEvent(SimpleEvent event) {
        return events.stream().filter(p -> event.equals(p.getValue())).count() > 0;
    }

    public List<Object> getEvents(Class<? extends ContextEvent> cls) {
        return events.stream().filter(p -> cls.isInstance(p.getValue())).map(Pair::getValue).collect(Collectors.toList());
    }

    public void doneWith(ContextEvent event) {
        events = events.stream().filter(p -> !event.equals(p.getValue())).collect(Collectors.toList());
    }

    public Long getHour() {
        return hour;
    }

    public void setHour(Long hour) {
        this.hour = hour;
    }

    public World.DwarfPoint getDwarfPosition(Dwarf dwarf) {
        WorldDwarfObjectRepository objectRepo = context.getBean(WorldDwarfObjectRepository.class);
        return objectRepo.findFirstByDwarf(dwarf).orElseThrow(RuntimeException::new);
    }

    public List<World.Something> getAllAround(World.Something self) {
        WorldObjectRepository pointRepo = context.getBean(WorldObjectRepository.class);
        List<World.Something> ret = new ArrayList<>();
        pointRepo.findAll(something.point.x.between(self.getPoint().getX() - LiveCycle.VISION_LENGTH, self.getPoint().getX() + LiveCycle.VISION_LENGTH).and(something.point.y.between(self.getPoint().getY() - LiveCycle.VISION_LENGTH, self.getPoint().getY() + LiveCycle.VISION_LENGTH)).and(something.world.id.eq(self.getWorld().getId()))).forEach(ret::add);
        return ret;
    }

    public boolean move(Dwarf dwarf, int x, int y) {
        WorldObjectRepository pointRepo = context.getBean(WorldObjectRepository.class);
        World.DwarfPoint old = getDwarfPosition(dwarf);
        try {
            World.DwarfPoint neo = new World.DwarfPoint(old.getWorld(), old.getDwarf(), old.getPoint().getX() + x, old.getPoint().getY() + y);
            Stream<World.Something> inPlace = StreamSupport.stream(pointRepo.findAll(something.point.x.eq(neo.getPoint().getX()).and(something.point.y.eq(neo.getPoint().getY()))).spliterator(), false);
            if (inPlace.filter(s -> Arrays.asList(WorldObjectType.DWARF).contains(s.getRef().getKey())).count() > 0)
                return false;
            pointRepo.save(neo);
            pointRepo.delete(old);
        } catch (OptimisticLockingFailureException e) {
            return false;
        }
        return true;
    }

    public boolean eat(Dwarf dwarf) {
        WorldObjectRepository pointRepo = context.getBean(WorldObjectRepository.class);
        World.DwarfPoint old = getDwarfPosition(dwarf);
        try {
            Iterable<World.Something> foodInPlace = pointRepo.findAll(something.point.x.eq(old.getPoint().getX()).and(something.point.y.eq(old.getPoint().getY())).and(something.point.type.eq(WorldObjectType.FOOD)));
            if (StreamSupport.stream(foodInPlace.spliterator(), false).count() == 0)
                return false;
            World.FoodPoint food = StreamSupport.stream(foodInPlace.spliterator(), false).findFirst().map(o -> (World.FoodPoint) o).orElseThrow(RuntimeException::new);
            pointRepo.delete(food);
        } catch (OptimisticLockingFailureException e) {
            return false;
        }
        return true;
    }
}
