package in.ocsf.frei.geld.core.model;/* kpmy 15.11.2017 */

import com.querydsl.core.annotations.QueryEntity;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.apache.commons.lang3.tuple.MutablePair;
import org.springframework.data.annotation.Id;
import org.springframework.data.annotation.Version;
import org.springframework.data.mongodb.core.mapping.DBRef;
import org.springframework.data.mongodb.core.mapping.Document;

import java.io.Serializable;


@Document(collection = "world")
public class World {

    public static final int defaultId = 42;

    @Id
    private Integer id = defaultId;

    @Version
    private Long _version;

    private Long hour;

    private Long day;

    public World() {
        this.hour = 0L;
        this.day = 0L;
    }

    public Long getHour() {
        return hour;
    }

    public void setHour(Long hour) {
        this.hour = hour;
    }

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public Long getDay() {
        return day;
    }

    public void setDay(Long day) {
        this.day = day;
    }

    @QueryEntity
    public static class WorldPoint2D implements Serializable {

        private int x;
        private int y;
        private WorldObjectType type;

        public WorldPoint2D(int x, int y, WorldObjectType type) {
            this.x = x;
            this.y = y;
            this.type = type;
        }

        public WorldPoint2D() {
        }

        public static WorldPoint2D fromString(String value) {
            String[] p = value.split("_", -1);
            return new WorldPoint2D(Integer.parseInt(p[1]), Integer.parseInt(p[2]), WorldObjectType.valueOf(p[0]));
        }

        public WorldObjectType getType() {
            return type;
        }

        public void setType(WorldObjectType type) {
            this.type = type;
        }

        public int getX() {
            return x;
        }

        public void setX(int x) {
            this.x = x;
        }

        public int getY() {
            return y;
        }

        public void setY(int y) {
            this.y = y;
        }

        @Override
        public String toString() {
            return type.name() + "_" + x + "_" + y;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;

            if (o == null || getClass() != o.getClass()) return false;

            WorldPoint2D that = (WorldPoint2D) o;

            return new EqualsBuilder()
                    .append(x, that.x)
                    .append(y, that.y)
                    .isEquals();
        }

        @Override
        public int hashCode() {
            return new HashCodeBuilder(17, 37)
                    .append(x)
                    .append(y)
                    .toHashCode();
        }
    }

    @Document(collection = "world-object")
    public static abstract class Something {

        @Id
        private WorldPoint2D point;

        @Version
        private Long _version;

        @DBRef
        private World world;

        public Something() {
            this.point = new WorldPoint2D(0, 0, WorldObjectType.EMPTY);
        }

        public World getWorld() {
            return world;
        }

        public void setWorld(World world) {
            this.world = world;
        }

        public WorldPoint2D getPoint() {
            return point;
        }

        public void setPoint(WorldPoint2D point) {
            this.point = point;
        }

        public abstract MutablePair<WorldObjectType, Object> getRef();
    }

    @Document(collection = "world-object")
    public static class DwarfPoint extends Something {

        @DBRef
        private Dwarf dwarf;

        public DwarfPoint() {
            super();
        }

        public DwarfPoint(World world, Dwarf dwarf, int x, int y) {
            this();
            this.dwarf = dwarf;
            this.setWorld(world);
            this.setPoint(new WorldPoint2D(x, y, WorldObjectType.DWARF));
        }

        public Dwarf getDwarf() {
            return dwarf;
        }

        public void setDwarf(Dwarf dwarf) {
            this.dwarf = dwarf;
        }

        @Override
        public MutablePair<WorldObjectType, Object> getRef() {
            return MutablePair.of(getPoint().getType(), dwarf.getId());
        }
    }

    public static class FoodPoint extends Something {

        public FoodPoint() {
            super();
        }

        public FoodPoint(World world, int x, int y) {
            this();
            this.setWorld(world);
            this.setPoint(new WorldPoint2D(x, y, WorldObjectType.FOOD));
        }

        @Override
        public MutablePair<WorldObjectType, Object> getRef() {
            return MutablePair.of(getPoint().getType(), "f00d");
        }
    }
}
