package in.ocsf.frei.geld.core.model.aspects;/* kpmy 25.11.2017 */

import in.ocsf.frei.geld.core.model.Dwarf;
import in.ocsf.frei.geld.core.model.DwarfContext;
import org.apache.commons.lang3.RandomUtils;

import java.util.*;
import java.util.function.BiConsumer;
import java.util.stream.Collectors;

public class DecisionAspect implements DwarfDefaultAspect {

    private Map<String, DecisionDescriptor> decisions;

    public DecisionAspect() {
    }

    public static DecisionAspect getInstance() {
        DecisionAspect ret = new DecisionAspect();
        ret.decisions = new HashMap<>();
        return ret;
    }

    public Map<String, DecisionDescriptor> getDecisions() {
        return decisions;
    }

    public void setDecisions(Map<String, DecisionDescriptor> decisions) {
        this.decisions = decisions;
    }

    private List<String> getDecision(DwarfTaskDescriptor task) {
        List<RunnerCode> ret = new ArrayList<>();
        int len = RandomUtils.nextInt(1, 20);
        while (len > 0) {
            ret.add(RunnerCode.values()[RandomUtils.nextInt(0, RunnerCode.values().length)]);
            len--;
        }
        return ret.stream().map(c -> c.getCode()).collect(Collectors.toList());
    }

    @Override
    public BiConsumer<DwarfContext, Dwarf> perform() {
        return (ctx, dwarf) -> {
            List<DecisionEvent> dl = (List) ctx.getEvents(DecisionEvent.class);
            dl.forEach(e -> {
                if (e instanceof ChangeExecutorEvent) {
                    ctx.doneWith(e);
                    Optional.ofNullable(decisions.get(((ChangeExecutorEvent) e).prev.getId())).filter(d -> Arrays.asList(ExecutorStatus.PENDING, ExecutorStatus.NONE).contains(d.getStatus())).ifPresent(d -> {
                        ctx.addEvent(this, new StopRunnerEvent(d));
                        d.setStatus(ExecutorStatus.BREAK);
                    });
                    List<String> code = getDecision(((ChangeExecutorEvent) e).curr);
                    DecisionDescriptor decision = new DecisionDescriptor();
                    decision.setCode(code);
                    decision.setStatus(ExecutorStatus.NONE);
                    decision.setId(((ChangeExecutorEvent) e).curr.getId());
                    ctx.addEvent(this, new StartRunnerEvent(decision));
                    decisions.put(decision.getId(), decision);
                }
            });
        };
    }

    public ExecutorStatus getStatus(String id) {
        return Optional.ofNullable(decisions.get(id)).map(DecisionDescriptor::getStatus).orElse(ExecutorStatus.NONE);
    }

    public void setStatus(String id, ExecutorStatus status) {
        Optional.ofNullable(decisions.get(id)).ifPresent(d -> d.setStatus(status));
    }
}
