package in.ocsf.frei.geld.core.model.aspects;/* kpmy 25.11.2017 */

import in.ocsf.frei.geld.core.model.Dwarf;
import in.ocsf.frei.geld.core.model.DwarfContext;
import org.apache.commons.math3.complex.Complex;

import java.util.Arrays;
import java.util.function.BiConsumer;
import java.util.function.BiFunction;

import static in.ocsf.frei.geld.core.model.aspects.ExecutorStatus.*;

public class ManagerAspect implements DwarfDefaultAspect {

    private static BiFunction<Complex, Complex, Boolean> deltaF = (r, l) -> Math.abs(r.abs() - l.abs()) > 0.2;
    private DwarfTaskExecutor executor;

    public static ManagerAspect getInstance() {
        ManagerAspect ret = new ManagerAspect();
        return ret;
    }

    public DwarfTaskExecutor getExecutor() {
        return executor;
    }

    public void setExecutor(DwarfTaskExecutor executor) {
        this.executor = executor;
    }

    @Override
    public BiConsumer<DwarfContext, Dwarf> perform() {
        return (ctx, dwarf) -> {
            if (executor == null) {
                executor = DwarfTaskExecutor.getInstance();
                executor.setDescriptor(TaskAspect.REST);
            }

            DwarfTaskDescriptor top = dwarf.getTasks().getQueue().values().stream().sorted((l, r) -> {
                Complex lc = l.getComplexValue();
                Complex rc = r.getComplexValue();
                return -Double.valueOf(lc.abs()).compareTo(Double.valueOf(rc.abs()));
            }).findFirst().orElse(TaskAspect.REST);

            if (top.getTask() != executor.getDescriptor().getTask()) {
                if (deltaF.apply(top.getComplexValue(), executor.getDescriptor().getComplexValue())) {
                    ctx.addEvent(this, new ChangeExecutorEvent(executor.getDescriptor(), top));
                    executor.setDescriptor(top);
                    executor.setHours(0);
                }
            } else if (!top.getComplexValue().equals(executor.getDescriptor().getComplexValue())) {
                top.setId(executor.getDescriptor().getId());
                executor.setDescriptor(top);
            }

            executor.setHours(executor.getHours() + 1);

            if (executor.getDescriptor().getTask() != DwarfTask.REST) {
                ExecutorStatus status = dwarf.getDecision().getStatus(executor.getDescriptor().getId());
                if (Arrays.asList(OK).contains(status))
                    dwarf.getEnergy().doSuccess(executor.getHours());
                if (Arrays.asList(ERROR).contains(status))
                    dwarf.getEnergy().doError(executor.getHours());
                if (Arrays.asList(OK, ERROR, BREAK).contains(status))
                    executor = null;
            } else {
                dwarf.getEnergy().doRest(executor.getHours());
            }
        };
    }
}
