package in.ocsf.frei.geld.core.model.aspects;/* kpmy 25.11.2017 */

import in.ocsf.frei.geld.core.model.Dwarf;
import in.ocsf.frei.geld.core.model.DwarfContext;
import in.ocsf.frei.geld.core.model.LiveCycle;

import java.util.ArrayList;
import java.util.List;
import java.util.function.BiConsumer;

import static in.ocsf.frei.geld.core.model.aspects.ExecutorStatus.*;

public class RunnerAspect implements DwarfDefaultAspect {

    private String id;

    private List<String> code;

    private int pos;

    private int left;

    private boolean error;

    public RunnerAspect() {

    }

    public static RunnerAspect getInstance() {
        RunnerAspect ret = new RunnerAspect();
        ret.left = LiveCycle.MAX_STEPS;
        ret.pos = -1;
        ret.code = new ArrayList<>();
        ret.error = false;
        return ret;
    }

    public List<String> getCode() {
        return code;
    }

    public void setCode(List<String> code) {
        this.code = code;
    }

    public int getPos() {
        return pos;
    }

    public void setPos(int pos) {
        this.pos = pos;
    }

    public int getLeft() {
        return left;
    }

    public void setLeft(int left) {
        this.left = left;
    }

    public boolean isError() {
        return error;
    }

    public void setError(boolean error) {
        this.error = error;
    }

    @Override
    public BiConsumer<DwarfContext, Dwarf> perform() {
        return (ctx, dwarf) -> {
            List<RunnerEvent> rl = (List) ctx.getEvents(RunnerEvent.class);
            rl.forEach(e -> {
                if (e instanceof StopRunnerEvent) {
                    pos = -1;
                    code = new ArrayList<>();
                    error = false;
                    id = null;
                } else if (e instanceof StartRunnerEvent) {
                    DecisionDescriptor descr = (((StartRunnerEvent) e).getDecision());
                    id = descr.getId();
                    code = descr.getCode();
                    pos = 0;
                    error = false;
                    dwarf.getDecision().setStatus(id, PENDING);
                }
            });
            left = Math.min(LiveCycle.MAX_STEPS, left + 8);
            boolean ok = true;
            while (!error && ok && pos >= 0 && pos < code.size()) {
                ok = false;
                RunnerCode op = RunnerCode.codeOf(this.code.get(pos));
                if (op.getCost() <= left) {
                    switch (op) {
                        case UP:
                            ok = ctx.move(dwarf, 0, 1);
                            break;
                        case DOWN:
                            ok = ctx.move(dwarf, 0, -1);
                            break;
                        case LEFT:
                            ok = ctx.move(dwarf, -1, 0);
                            break;
                        case RIGHT:
                            ok = ctx.move(dwarf, 1, 0);
                            break;
                        case NOOP:
                            ok = true;
                            break;
                        case EAT:
                            if (ctx.eat(dwarf))
                                dwarf.getEnergy().addFood();
                            ok = true;
                            break;
                        default:
                            throw new RuntimeException("unknown code " + op + " at pos" + pos);
                    }
                    //Logger.getLogger(getClass()).info("op "+op+" at "+pos+"/"+code.size()+ " is "+ok);
                    if (ok) {
                        pos++;
                        left -= op.getCost();
                        if (pos == code.size()) {
                            dwarf.getDecision().setStatus(id, OK);
                        }
                    } else {
                        error = true;
                        dwarf.getDecision().setStatus(id, ERROR);
                    }
                } else {
                    ok = false;
                }
            }
        };
    }
}
