package in.ocsf.frei.geld.core.service;/* kpmy 15.11.2017 */

import in.ocsf.frei.geld.core.model.Dwarf;
import in.ocsf.frei.geld.core.model.DwarfState;
import in.ocsf.frei.geld.core.model.LiveCycle;
import in.ocsf.frei.geld.core.model.World;
import in.ocsf.frei.geld.core.repo.DwarfRepository;
import in.ocsf.frei.geld.core.repo.WorldRepository;
import org.apache.log4j.Logger;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

@Service
public class LiveCycleService {

    @Inject
    private DwarfRepository dwarfRepo;

    @Inject
    private WorldRepository worldRepo;

    @Inject
    private DwarfService dwarfService;

    private Logger log = Logger.getLogger(getClass());

    @Scheduled(fixedDelay = 1000L)
    public void run() {
        IntStream.rangeClosed(1, LiveCycle.DAYS).forEach(day -> {
            log.info("day " + day);
            IntStream.range(0, LiveCycle.HOURS).forEach(hour -> {
                World world = worldRepo.findOne(World.defaultId);
                List<Dwarf> dwarves = new ArrayList<>();
                dwarves.addAll(dwarfRepo.findAllByStateInAndWorld(new DwarfState[]{DwarfState.old, DwarfState.mature, DwarfState.child}, world).collect(Collectors.toList()));
                try {
                    dwarves.parallelStream().peek(dwarf -> runDwarfHour(world, hour, dwarf)).forEach(dwarf -> dwarfRepo.save(dwarf));
                } catch (Exception e) {
                    throw new RuntimeException(e);
                }
                world.setHour(world.getHour() + 1);
                worldRepo.save(world);
            });
            World world = worldRepo.findOne(World.defaultId);
            world.setDay(world.getDay() + 1);
            worldRepo.save(world);
        });
    }

    private void runDwarfHour(World world, Integer hour, Dwarf dwarf) {
        if (hour == 0) {
            if (dwarf.getDay() == LiveCycle.DAYS) {
                dwarf.setDay(1);
                dwarf.setAge(dwarf.getAge() + 1);
            } else {
                dwarf.setDay(dwarf.getDay() + 1);
            }
            if (dwarf.getAge() >= dwarf.getDeath()) {
                dwarf.setState(DwarfState.dead);
            }
        }
        if (!dwarf.getState().equals(DwarfState.dead)) {
            dwarfService.runDwarfHour(world, dwarf);
        }
        if (dwarf.getState().equals(DwarfState.dead)) {
            log.info("dwarf " + dwarf.getName() + " dead");
        } else {
            dwarf.setHour(hour);
            if (dwarf.getAge() > 45) {
                dwarf.setState(DwarfState.old);
            } else if (dwarf.getAge() >= 18) {
                dwarf.setState(DwarfState.mature);
            }
        }
    }
}
